module Ventiuna
	class Hand
		attr_accessor :cards

		def initialize(card=[], *other_cards)
			@cards = Array(card) + other_cards
			@soft = false
			@active = true
		end

		def << (another_card)
			self.cards << another_card
		end
		alias :hit :<<

		def stand
			@active = false
		end

		def split
			return false unless self.pair?
			[Ventiuna::Hand.new(self.cards.first), Ventiuna::Hand.new(self.cards.last)]
		end

		def value
			sum = 0
			self.cards.sort.each_with_index do |card, i|
				# hit low if haven't reached 11 or there are more cards left 
				hit_low = (sum >= 11) || (i+1 != self.cards.size)
				@soft = card.ace? && !hit_low
				sum += card.value(low: hit_low)
			end
			sum
		end

		def active?
			@active
		end

		def blackjack?
			self.value == 21 && self.cards.size == 2
		end

		def bust?
			self.value > 21 #&& !self.soft?
		end

		def pair?
			self.cards.size == 2 && (self.cards.first.value == self.cards.last.value)
		end

		def soft?
			self.value
			@soft
		end

		def to_s
			"#{self.cards.collect{|c| c.to_s}} = #{self.value}"
		end

		def to_db
			s = "#{value}"
			s += "I" if cards.size == 2
			s += "S" if soft?
			s += "P" if pair?
			s
		end
	end
end