module Ventiuna
	class Player
		attr_accessor :name, :balance, :hands, :bet, :wins, :loses, :ties
		attr_reader :decisions

		def initialize(name, strategy: "Learning", balance: 1000000.0)
			@name = name
			@strategy = Ventiuna::Strategy.find_or_create_by_name(strategy)
			@balance = balance
			@hands = [Ventiuna::Hand.new]
			@bet = 0
			@wins = 0
			@loses = 0
			@ties = 0
			@decisions = Hash.new([])
		end

		def stats
			"Wins: #{self.wins} (#{self.wins_percent.round}%),    Loses: #{self.loses} (#{self.loses_percent.round}%),    Ties: #{self.ties} (#{self.ties_percent.round}%)"
		end

		def wins_percent
			return 0 if total_rounds == 0
			(self.wins.to_f/self.total_rounds) * 100
		end

		def loses_percent
			return 0 if total_rounds == 0
			(self.loses.to_f/self.total_rounds) * 100
		end

		def ties_percent
			return 0 if total_rounds == 0
			(self.ties.to_f/self.total_rounds) * 100
		end

		def total_rounds
			@wins+@loses+@ties
		end

		def place_bet(min_bet, max_bet)
			@bet = min_bet
			@balance -= @bet
		end

		def active_hand
			hands.select{ |h| h.active? }.first
		end
		alias :hand :active_hand

		def reset_hands
			@hands = [Ventiuna::Hand.new]
		end
		alias :reset_hand :reset_hands

		def decision(dealers_card, strategy: "interactive")
			define_posible_moves(dealers_card)
			can_split = hands.size == 1
			#d = @strategy.decision(dealers_card, hand, can_split)
			d = nil
			
			unless d
				d = "s"
				case strategy
				when "random"
					d = random_strategy
				when "interactive"
					d = interactive
				end
			end
			move = Ventiuna::Move.find_by_strategy_id_and_dealers_card_value_and_hand_and_decision(@strategy.id, dealers_card.value, hand.to_db, d)
			#puts move.inspect
			@decisions[hand] = decisions[hand] << move
			#puts @decisions
			d
		end

		def reset_decisions
			@decisions = Hash.new([])
		end

		private
		def posible_moves(dealers_card)
			moves_list = ["s"]
			return moves_list if active_hand.bust?

			moves_list.unshift("h") if active_hand.soft? || active_hand.value < 21
			moves_list.unshift("sp") if active_hand.pair?
			moves_list.unshift("d") if active_hand.cards.size == 2
			
			moves_list
		end

		def define_posible_moves(dealers_card)
			posible_moves(dealers_card).each do |m|
				Ventiuna::Move.find_or_create_by_strategy_id_and_dealers_card_value_and_hand_and_decision(@strategy.id, dealers_card.value, hand.to_db, m).save
			end
		end

		def random_strategy
			return "s" if self.active_hand.bust?
			return "s" if self.active_hand.value == 21 || self.active_hand.blackjack?
			return ["s", "h", "d", "sp"].shuffle.pop if self.active_hand.pair?
			return ["s", "h", "d"].shuffle.pop if self.active_hand.cards.size == 2
			["s", "h"].shuffle.pop
		end

		def interactive
			choice = $stdin.gets.to_s.strip

			case choice
			when /^hit$|^h$/i
				"h"
			when /^stand$|^s$/i
				"s"
			when /^double$|^d$/i
				"d"
			when /^split$|^sp$/i
				"sp"
			else
				puts "invalid choice!"
				interactive()
			end
				
		end
	end
end